@@ -1,6 +1,6 @@
 # abstract_tool.py
 #
-# Copyright 2018-2020 Romain F. T.
+# Copyright 2018-2021 Romain F. T.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -57,14 +57,23 @@ class AbstractAbstractTool():
 	def add_tool_action_simple(self, action_name, callback):
 		"""Convenient wrapper method adding a stateless action to the window. It
 		will be named 'action_name' (string) and activating the action will
-		trigger the method 'callback'.""" # XXX allow to set shortcuts ?
+		trigger the method 'callback'."""
+		# TODO allow to set shortcuts here
 		self.window.add_action_simple(action_name, callback, None)
 
 	def add_tool_action_boolean(self, action_name, default):
-		self.window.options_manager.add_tool_option_boolean(action_name, default)
+		self.window.options_manager.add_option_boolean(action_name, default)
 
 	def add_tool_action_enum(self, action_name, default):
-		self.window.options_manager.add_tool_option_enum(action_name, default)
+		self.window.options_manager.add_option_enum(action_name, default)
+
+	def load_tool_action_boolean(self, action_name, key_name):
+		om = self.window.options_manager
+		return om.add_option_from_bool_key(action_name, key_name)
+
+	def load_tool_action_enum(self, action_name, key_name):
+		om = self.window.options_manager
+		return om.add_option_from_enum_key(action_name, key_name)
 
 	def get_option_value(self, action_name):
 		return self.window.options_manager.get_value(action_name)
@@ -79,7 +88,7 @@ class AbstractAbstractTool():
 		self.set_action_sensitivity('cairo_operator', self.use_operator)
 
 	def get_settings(self):
-		return self.window._settings
+		return self.window.options_manager._tools_gsettings
 
 	############################################################################
 	# Various utilities ########################################################
@@ -109,11 +118,18 @@ class AbstractAbstractTool():
 		tools_menu.append(self.label, 'win.active_tool::' + self.id)
 
 	def get_options_model(self):
+		"""Returns a Gio.MenuModel corresponding to the tool's options. It'll be
+		shown in the menubar (if any) and in the bottom pane (if the tool's
+		bottom pane supports such a feature)."""
 		fpath = self.UI_PATH + 'tool-' + self.id + '.ui'
 		builder = Gtk.Builder.new_from_resource(fpath)
 		return builder.get_object('options-menu')
 
 	def get_options_widget(self):
+		"""Returns a Gtk.Widget (normally a box) corresponding to the tool's
+		options. It'll be in the bottom pane (if the tool's bottom pane supports
+		such a feature) in replacement of the Gio.MenuModel if such a simple
+		menu can't provide all the features."""
 		return None
 
 	def get_edition_status(self):
@@ -130,7 +146,7 @@ class AbstractAbstractTool():
 		                                                tooltip_text=self.label)
 		self.row.set_detailed_action_name('win.active_tool::' + self.id)
 		self.label_widget = Gtk.Label(label=self.label) #, use_underline=True)
-		if self.get_settings().get_boolean('big-icons'):
+		if self.window.gsettings.get_boolean('big-icons'):
 			size = Gtk.IconSize.LARGE_TOOLBAR
 		else:
 			size = Gtk.IconSize.SMALL_TOOLBAR
@@ -150,7 +166,7 @@ class AbstractAbstractTool():
 
 	def update_icon_size(self):
 		image = self.row.get_children()[0].get_children()[0]
-		if self.get_settings().get_boolean('big-icons'):
+		if self.window.gsettings.get_boolean('big-icons'):
 			size = Gtk.IconSize.LARGE_TOOLBAR
 		else:
 			size = Gtk.IconSize.SMALL_TOOLBAR
@@ -193,6 +209,8 @@ class AbstractAbstractTool():
 		self._ongoing_operation = True
 
 	def apply_operation(self, operation):
+		"""Complete method to apply an operation: the operation is applied and
+		the image is updated as well as the state of actions."""
 		self.simple_apply_operation(operation)
 		self.get_image().update_actions_state()
 		self.get_image().update_history_sensitivity()
@@ -201,11 +219,11 @@ class AbstractAbstractTool():
 		"""Simpler apply_operation, for the 'rebuild from history' method."""
 		try:
 			self.do_tool_operation(operation)
+			self.get_image().add_to_history(operation)
 		except Exception as e:
 			self.show_error(str(e))
 		self._ongoing_operation = False
-		self.get_image().add_to_history(operation)
-		self.non_destructive_show_modif()
+		self.non_destructive_show_modif() # XXX nécessaire ?
 
 	############################################################################
 	# Selection ################################################################
